%RESTART_STEP_NAVIER restart unsteady flow in step domain
%   IFISS scriptfile: DJS; 19 November 2014.
% Copyright (c) 2014 D.J. Silvester
clear all
load step_unsteadyflow.mat
tbegin=time(end);
global pde 
fprintf('Restarted flow in step domain ... \n')
global viscosity
%
%% initialize for time stepping iteration 
%% load assembled matrices
gohome
cd datafiles
load step_stokes_nobc.mat
%
%
%% set parameters
pde=14; domain=3;
fprintf('Discrete Saddle-Point DAE system ...\n')
oldtfinal=tbegin; oldufinal=U(:,end); oldudotfinal=Udot(:,end);
tfinal = default('new final time? (150 more units)',tbegin+150);
tol = default('accuracy tolerance? (default 1e-4)',1e-4);
nstar = default('averaging frequency? (default 10)',10);
dtzero=1e-9; uzero=oldufinal; udotzero=oldudotfinal; gzero=zeros(size(g));
%
if qmethod>1 & exist('unstablemode.mat','file')==2,
    load unstablemode.mat dpsi
    [xypp,mp,map] = q2q1map(xy,mv,0);
    solplot(dpsi,xypp,x,y,99), title('perturbation field'), shg
    addpert = default('add perturbation to flow solution? enter 0/1 (yes)',1);
   if addpert==1, fprintf('norm of perturbation is %g \n',norm(dpsi,inf)), end
else
   addpert = 0;
end

%% compute solution
tstart = tic;
if qmethod>1, 
np=length(g); 
AxB='defaultAxB';
   if addpert,
   [DT,U,Udot,xtime] = xstabtrNS(qmethod,xy,mv,bound,A,B,sparse(np,np),G,AxB,...
							uzero,udotzero,dtzero,tbegin,tfinal,tol,nstar,1,dpsi);
   else
   [DT,U,Udot,xtime] = xstabtrNS(qmethod,xy,mv,bound,A,B,sparse(np,np),G,AxB,...
                              uzero,udotzero,dtzero,tbegin,tfinal,tol,nstar,1);
   end
else  
% call the specialized saddle point system solver 
AxB='colamdAxB'; 
% fixed beta (independent of the viscosity)    
beta=1;   
if qmethod==1,   
	  beta=1/4;     % default parameter
end  
% scaled beta (mutiplied by the viscosity)   
beta=beta*viscosity;
[DT,U,Udot,xtime] = xstabtrNS(qmethod,xy,ev,bound,A,B,beta*C,G,AxB,...
							uzero,udotzero,dtzero,tstart,tfinal,tol,nstar,1);
end
etoc=toc(tstart); fprintf('Integration took  %8.3e seconds\n\n',etoc) 

save restarted_obstacle_unsteadyflow.mat DT U Udot xtime viscosity
%
% visualise solution and hold the timestep sequence plot
marker = 'bo'; %'bo'
dttplot(DT,66,marker)
%pause(3)
%
% plot mean vorticity
fprintf('plotting mean vorticity  evolution ...')
meanv = meanvorticityplot(qmethod,mv,U,xtime,By,Bx,G,xy,102,0);
%
showsol = default('display flow solution? enter 0/1 (no)',0);
if showsol,
if qmethod>1,
   obstacle_unsteadyflowplot(qmethod,mv,U,xtime,By,Bx,G,xy,xyp,x,y,...
                             bound,bndxy,bnde,obs,0.01,166);
else
   obstacle_unsteadyflowplot(qmethod,ev,U,xtime,By,Bx,G,xy,xyp,x,y,...
                             bound,bndxy,bnde,obs,0.01,166); end
end
%%
fprintf('... to generate a movie run step_flowmovie\n')
fprintf('    see help step_flowmovie ...\n')
%help step_flowmovie
%%% compute divergence error
if qmethod>1
   error_div = q2div(xy,mv,[U(:,end);gzero]);	
else
   error_div = q1div(xy,ev,[U(:,end);gzero]);	 	
end
return
