%UNSTEADY_STEP_NAVIER solve Navier-Stokes problem in step domain
%   IFISS scriptfile: DJS; 22 August 2015
% Copyright (c) 2009 D.J. Silvester
clear variables
global pde 
fprintf('Unsteady flow in a step domain ...\n')
global viscosity
viscosity=default('viscosity parameter (default 1/50)',1/50);
%
%% initialize for time stepping iteration: compute Stokes solution
%% load assembled matrices
gohome
cd datafiles
load step_stokes_nobc.mat
%
%
pde=14; domain=3;
%% set parameters
fprintf('Discrete Saddle-Point DAE system ...\n')
tfinal = default('target time? (default 500)',500);
tol = default('accuracy tolerance? (default 1e-4)',1e-4);
nstar = default('averaging frequency? (default 10)',10);
dtzero=1e-9; uzero=zeros(size(f));gzero=zeros(size(g));
%% compute solution
tstart = tic;
if qmethod>1, 
np=length(g); 
AxB='defaultAxB'; 
[DT,U,Udot,time] = stabtrNS(qmethod,xy,mv,bound,A,B,sparse(np,np),G,AxB,...
							uzero,dtzero,tfinal,tol,nstar,1);
else  
% call the specialized saddle point system solver 
AxB='colamdAxB';  
beta=1;   
if qmethod==1, 
	  beta=1/4;     % default parameter
end  
% scaled beta (mutiplied by the viscosity)   
beta=beta*viscosity;
[DT,U,Udot,time] = stabtrNS(qmethod,xy,ev,bound,A,B,beta*C,G,AxB,...
							uzero,dtzero,tfinal,tol,nstar,1);   
end
etoc=toc(tstart); fprintf('Integration took  %8.3e seconds\n\n',etoc)

save step_unsteadyflow.mat DT U Udot time viscosity
%
% visualise solution and hold the timestep sequence plot
marker = 'bo';
dttplot(DT,13,marker)
pause(3)
%
% plot mean vorticity
fprintf('plotting mean vorticity  evolution .. this takes time ...')
meanv = meanvorticityplot(qmethod,mv,U,time,By,Bx,G,xy,101,0);
%
showsol = default('display flow solution? enter 0/1 (yes)',1);
if showsol
symm = default('Sanity check: is the step symmmetric? enter 0/1 (yes)',1);
if qmethod>1,
step_unsteadyflowplot(qmethod,mv,U,time,By,Bx,G,xy,xyp,x,y,bound,0.01,199,symm);
else
step_unsteadyflowplot(qmethod,ev,U,time,By,Bx,G,xy,xyp,x,y,bound,0.01,199,symm);
end
end


%%% compute divergence error
if qmethod>1
   error_div = q2div(xy,mv,[U(:,end);gzero]);	
else
   error_div = q1div(xy,ev,[U(:,end);gzero]);	 	
end
return
